
# frozen_string_literal: true

module Hermes
  class EligibleCountOffer
    include Hermes::Concerns::CommonFieldsConcern

    field :offer_status, type: String
    field :reserved_entitites_eligible_request_nb, type: Hash

    belongs_to(:housing_offer, class_name: 'Hermes::HousingOffer', index: true)

    common_indexes *%i[
      updated_at
      offer_status
    ]

    one_week = 3600*24*7
    index({ created_at: 1 }, { expire_after_seconds: one_week})

    before_create :calculate_count, :update_offer_status

    writer_roles 'Superadmin', 'Administrator'
    reader_roles 'Manager'


    def calculate_count
      return unless offer = self.housing_offer

      self.reserved_entitites_eligible_request_nb = {}

      offer.active_reservations.each do |reserved_entity|
        self.reserved_entitites_eligible_request_nb[reserved_entity] = {}

        ['department', 'seeked_bordering', 'all_eligibility'].each do |type|
          hash_key_type = type 
          query = if type == 'all_eligibility'
              hash_key_type = 'total'
              Hermes::HousingRequest.where(offer.all_eligibility_query)
            else
              offer.eligibles_requests(type)
          end 

          count = query.where(entity_id: reserved_entity).count(hint: Hermes::HousingOffer.find_hint(type))
          self.reserved_entitites_eligible_request_nb[reserved_entity][hash_key_type] = count
        end
      end
    end

    def update_offer_status
      self.offer_status = self.housing_offer&.offer_status
    end 

  end
end
